{% load media %}
<!doctype html>

<head>
    <meta charset=utf-8>
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
    <meta name="viewport" content="width=device-width,initial-scale=1">

    <title>{% block title %}{{ page_title }}{% endblock %}</title>
    {% block favicon %}{% favicon url='favicon.png?{{ codename }}' %}{% endblock %}

    {% block icons %}
        {% stylesheet url='fontawesome/css/font-awesome.min.css' %}
    {% endblock %}

    {% block scripts %}
        {% script url='moment/min/moment-with-locales.min.js' %}
        {% script url='jquery/dist/jquery.min.js' %}
        {% script url='jquery-query-object/jquery.query-object.js' %}
        {% script url='underscore/underscore-min.js' %}
        {% script url='bootstrap/dist/js/bootstrap.min.js' %}
    {% endblock %}

    {% block media %}
        <script>var django = {jQuery: $}</script>
        {{ media }}
    {% endblock %}

    {% block bootstrap %}
        {% stylesheet url='bootstrap/dist/css/bootstrap.min.css' %}
    {% endblock %}

    {% stylesheet url='styles/era.css' %}
    {% block design %}{% endblock %}

    <!--[if lt IE 9]>
      <script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
    <![endif]-->   
</head>
<body class="{{ view.about }}">
    {% block body %}{% endblock %}
</body>
